//-----------------------------------------------------------------------------
//
//	This program is free software; you can redistribute it and/or
//	modify it under the terms of the GNU General Public License
//	as published by the Free Software Foundation; either version 2
//	of the License, or (at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//-----------------------------------------------------------------------------

// bootloader client for AVR USB DFU
// Javier Almansa Sobrino (javieralso@gmail.com)

#ifndef __BOOT__

#define __BOOT__

#define FALSE 0
#define TRUE 1

typedef unsigned char byte; /* Tipo byte */

typedef struct {

  byte bStatus;
  byte bwPollTimeOut[3];
  byte bState;
  byte iString;

} status; /* DFU_GETSTATUS response */

#define OK 0x00 /* No error condition is present */
#define errTARGET 0x01 /* File is not targeted for use by this device */
#define errFILE  0x02 /* File is for this device but fails some vendor-specific verification test */
#define errWRITE 0x03 /* Device id unable to write memory */
#define errERASE 0x04 /* Memory erase function failed */
#define errCHECK_ERASED 0x05 /* Memory erase check failed */
#define errPROG 0x06 /* Program memory function failed */
#define errVERIFY 0x07 /* Program memory verification failed */
#define errADDRESS 0x08 /* Cannot program memory due to received address that is out of range */
#define errNOTDONE 0x09 /* Received DFU_DNLOAD with wLegth = 0, but device does not think it has all the data yet */
#define errFIRMWARE 0x0A /* Device's firmware is corrupted . It cannot return to run-time operations */
#define errVENDOR 0x0B /* iString indicates a vendor-specific error */
#define errUSBR 0x0C /* Device detected unexpected USB reset signaling */
#define errPOR 0x0D /* Device detected unexpected power on reset */
#define errUNKNOWN 0x0e /* Something went wrong, but the device does not know what it was */
#define errSTALLEDPK 0x0f /* Device stalled an unexpected request */


/* Posibles errores */

#define appIDLE 0 /* Device is running its normal application */
#define appDETACH 1 /* Device is running its normal application, has received the DFU_DETACH
		       request and is waiting for a USB reset */
#define dfulDLE 2 /* Device is oprating in the DFU mode and is waiting for request */
#define dfuDNLOAD_SYNC 3 /* Device has received a block and is waiting for the host to
			    solicit the status via DFU_GETSTATUS */
#define dfuDNBUSY 4 /* Device is programming a control-write block into its non volatile memories */
#define dfuDNLOAD_IDLE 5 /* Device is processing a download operation. Expecting DFU_DNLOAD request */
#define dfuMANIFEST_SYNC 6 /* Device has received the final block of firmware from the host and is
			      waiting for receipt of DFU_GETSTATUS to begin the manifestation phase
			      or device has completed the manifestation phase and is waiting for receipt
			      of DFU_GETSTATUS */
#define dfuMANIFEST 7 /* Device is in the manifest phase */
#define dfuMANIFEST_WAIT_RESET 8 /* Device has programmed its memories and is waiting of a USB request or a power on reset */
#define dfuUPLOAD_IDLE 9 /* The device is processing an upload operation. Expecting DFU_UPLOAD request */
#define dfuError 10 /* An error has ocurred. Awaiting the DFU_CLRSTATUS request */

/* Valores para bState */

#define DFU_DETACH 0
#define DFU_DNLOAD 1
#define DFU_UPLOAD 2
#define DFU_GETSTATUS 3
#define DFU_CLRSTATUS 4
#define DFU_GETSTATE 5
#define DFU_ABORT 6

/* DFU Class-specific requests */


#define Id_prog_start 0x01
#define InitFlashProg 0x00
#define InitEEPROMProg 0x01

#define Id_display_data 0x03
#define DisplayFlashData 0x00
#define BlankCheckInFlash 0x01
#define DisplayEEPROMData 0x02

#define Id_write_command 0x04
#define FullChipErase 0x00

#define Id_read_command 0x05

#define Id_change_base_address 0x06

//Frames from host

#endif
