//-----------------------------------------------------------------------------
//
//	This program is free software; you can redistribute it and/or
//	modify it under the terms of the GNU General Public License
//	as published by the Free Software Foundation; either version 2
//	of the License, or (at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//-----------------------------------------------------------------------------

// bootloader client for AVR USB DFU
// Javier Almansa Sobrino (javieralso@gmail.com)

#include "bootAT90USB1287.h"
#include "bootUsb.h"
#include "hex.h"
#include "bootFunc.h"

void checkStatus (status *est, usb_dev_handle *d) {

  request peticion;

  peticion.bmRequestType = USB_ENDPOINT_IN+USB_TYPE_CLASS+USB_RECIP_INTERFACE;
  peticion.bRequest = DFU_GETSTATUS;
  peticion.wValue = 0;
  peticion.wIndex = 0;
  peticion.wLength = 6;
  
  usbRequest (d, peticion, (byte *)est);
  /* Envia el comando de peticion de estado */
}

int writeProg (FILE *fp, usb_dev_handle *d) {

  byte datos[65536], bStateError;
  int type, c;
  request peticion;
  status estado;
  unsigned char dataLoad[50];
  unsigned short addr, endAddr;
  unsigned char chk;
  unsigned short len, totalData = 0;
  char startAddrSet = FALSE;
  int index = 32, dataSend = 0;


  printf ("loading...\n");
  
  for (c = 0; c < 50; c++) {
    dataLoad[c] = 0xffu;
  }

  for (c == 0; c < 176; c++) {
    datos[c] = 0x00u;
  }

  peticion.bmRequestType = USB_ENDPOINT_OUT+USB_TYPE_CLASS+USB_RECIP_INTERFACE;
  peticion.bRequest = DFU_DNLOAD;
  peticion.wValue = 128u;
  peticion.wIndex = 0;
  peticion.wLength = 48u;
  
  datos[0] = Id_prog_start;
  datos[1] = 0x00u;
  
  while ((type = hex_read (fp, dataLoad, &addr, &chk, &len)) != 1) {
    if (type < 0) {
      printf ("parsing error: %i\n", type);
      //Error.
      return type;
    } else {
      if (type == 0u) {
	
	/* Linea de datos estandard */
	
	if (!startAddrSet) {
	  datos[2] = addr>>8;
	  datos[3] = addr;
	  /* Direccion de inicio */
	  startAddrSet = TRUE;
	}
	endAddr = addr + len - 1;
	/* Presunta direccion de fin */
	for (c = 0; c < len; c++) {
	  datos[index++] = dataLoad[c];
	  totalData++;
	}	
      } else {
	if (type == 2) {
	  /* Extended Segment address record */
	  if (totalData != 0u) {
	    /* Antes de cambiar de pagina hay que enviar datos */
	    if ((bStateError = sendData (d, totalData, endAddr, datos, &peticion, &dataSend)) != 0) {
	      printf ("bState error: 0x%02x\n", bStateError);
	    }
	    /* Envia los datos que faltan */
	    sendZLP (d);
	    checkStatus (&estado, d);
	    if (estado.bState != 0) {
	      printf ("bState error: 0x%02x\n", estado.bState);
	      return estado.bState;
	      /* Si se produce algun error se retorna */
	    }
	    totalData = 0;
	    index = 32;
	    startAddrSet = FALSE;
	  }
	  setPage (addr>>12, d);
	  	} else {
	  if (type == 3) {
	    /* Inicia el programa */
	    if (totalData != 0u) {
	      /* Antes de cambiar de pagina hay que enviar datos */
	      if ((bStateError = sendData (d, totalData, endAddr, datos, &peticion, &dataSend)) != 0) {
		printf ("bState error: 0x%02x\n", bStateError);
	      }
	      /* Envia los datos que faltan */
	      checkStatus (&estado, d);
	      if (estado.bState != 0) {
		printf ("bState error: 0x%02x\n", estado.bState);
		return estado.bState;
		/* Si se produce algun error se retorna */
	      }
	      sendZLP (d);
	      totalData = 0;
	      index = 32;
	      startAddrSet = FALSE;
	    }
	    printf ("Device programed\n%i Bytes sent\nStarting firmware...\n", dataSend);
	    startApp (addr, d);
	    printf ("Firmware started at address 0x%04x\n");
	    fclose (fp);
	    exit (0);
	  }
	}
      }
    }
    if (type == 0 && totalData == 128) {
      /* Ya tenemos un bloque para enviar */
      if ((bStateError = sendData (d, totalData, endAddr, datos, &peticion, &dataSend)) != 0) {
	printf ("bState error: 0x%02x\n", bStateError);
      }
      /* Envia los datos que faltan */
      totalData = 0;
      index = 32;
      startAddrSet = FALSE;
    }
  }
  if (totalData != 0u) {
    /* Antes de cambiar de pagina hay que enviar datos */
    if ((bStateError = sendData (d, totalData, endAddr, datos, &peticion, &dataSend)) != 0) {
      printf ("bState error: 0x%02x\n", bStateError);
    }
    /* Envia los datos que faltan */
    totalData = 0;
    index = 32;
    startAddrSet = FALSE;
  }
  sendZLP (d);

  checkStatus (&estado, d);

  if (estado.bState != 0) {
    printf ("bState error: 0x%02x\n", estado.bState);
    return estado.bState;
    /* Si se produce algun error se retorna */
  } else {
    printf ("Programming finished\n%i Bytes sent\n", dataSend);
  }
  return 0;
}

byte sendData (usb_dev_handle *d, unsigned short totalData, unsigned short endAddr, byte *datos, request *peticion, int *dataSend) {

  status estado;
  
  datos[4] = endAddr>>8;
  datos[5] = endAddr;
  peticion->wLength = 48 + totalData;
  
  *dataSend += (usbRequest (d, *peticion, datos) - 48);
  checkStatus (&estado, d);
  
  return estado.bState;
  
}

void startApp (unsigned short addr, usb_dev_handle *d) {

  /* 
     Inicia la aplicacion ejecutando un salto a addr
  */

  request peticion;
  byte comando[5];

  peticion.bmRequestType = USB_ENDPOINT_OUT+USB_TYPE_CLASS+USB_RECIP_INTERFACE;
  peticion.bRequest = DFU_DNLOAD;
  peticion.wValue = 0;
  peticion.wIndex = 0;
  peticion.wLength = 5;

  comando[0] = Id_write_command;
  comando[1] = 0x03u;
  comando[2] = 0x01u;
  comando[3] = addr>>16;
  comando[4] = addr>>8;
  
  usbRequest (d, peticion, comando);
}

void setPage (byte page, usb_dev_handle *d) {

  /*
    Cambia la pagina del dispositivo sobre la que 
    se operara.
  */

  request peticion;
  byte comando[4];

  peticion.bmRequestType = USB_ENDPOINT_OUT+USB_TYPE_CLASS+USB_RECIP_INTERFACE;
  peticion.bRequest = DFU_DNLOAD;
  peticion.wValue = 0;
  peticion.wIndex = 0;
  peticion.wLength = 4;

  comando[0] = Id_change_base_address;
  comando[1] = 0x03u;
  comando[2] = 0x00u;
  comando[3] = page;

  usbRequest (d, peticion, comando);

  sendZLP(d);
}

/* void readDevice (usb_dev_handle *d) { */

/*   /\* */
/*     Sin terminar. Se hicieron pruebas de lectura */
/*     pero la funcion no esta acabada. Se encuentra "desordenada". */
/*   *\/ */

/*   request peticion; */
/*   unsigned char datos[70000]; */
/*   int c; */


/*   FILE *fp; */
/*   fp = fopen ("salida.hex", "rw"); */

/*   datos[0] = Id_display_data; */
/*   datos[1] = 0x00; */
/*   datos[2] = 0x00; */
/*   datos[3] = 0x00; */
/*   datos[4] = 0x01; */
/*   datos[5] = 0x8f; */

/*   peticion.bmRequestType = USB_ENDPOINT_OUT+USB_TYPE_CLASS+USB_RECIP_INTERFACE; */
/*   peticion.bRequest = DFU_DNLOAD; */
/*   peticion.wValue = 0u; */
/*   peticion.wIndex = 0; */
/*   peticion.wLength = 6; */

/*   usbRequest (d, peticion, datos);  */
/*   /\* Peticion de lectura *\/ */

/*   sendZLP(d); */

/*   int a, b; */

/*   unsigned int addr = 0u; */
/*   printf ("uffff\n"); */
  
/*   peticion.bmRequestType = USB_ENDPOINT_IN+USB_TYPE_CLASS+USB_RECIP_INTERFACE; */
/*   peticion.bRequest = DFU_UPLOAD; */
/*   peticion.wValue = 0u; */
/*   peticion.wIndex = 0u; */
/*   peticion.wLength = 432; */
  
/*   while (usbRequest (d, peticion, datos) != 0) { */

/*     for (a=0; a < 384; a++) { */
/*       printf ("0x%02x - ", datos[a]); */
/*     } */
/*   } */
/*   printf ("\n"); */
/*   fclose (fp); */
/*   sendZLP (d); */

/* } */

int eraseWhole (usb_dev_handle *d) {
  
  /*
    Borra por completo el microcontrolador y realiza un chequeo de blanqueo.
    Si ha sido correctamente borrada devuelve 0.
    Si el error se ha producido al borrar, se devuelve -1.
    Si el error se ha producido en el chequeo de blanco, se devuelve -2.
  */
  
  request peticion;
  status estado;
  byte datos[6];
  
  peticion.bmRequestType = USB_ENDPOINT_OUT+USB_TYPE_CLASS+USB_RECIP_INTERFACE;
  peticion.bRequest = DFU_DNLOAD;
  peticion.wValue = 0;
  peticion.wIndex = 0;
  peticion.wLength = 3;
  
  datos[0] = Id_write_command;
  datos[1] = 0x00;
  datos[2] = 0xFF;
  
  usbRequest (d, peticion, datos); 
  /* Envia el comando de borrado */
     
  checkStatus (&estado, d);  

  if (estado.bStatus != OK) {
    return -1;
  } else {
    
    peticion.bmRequestType = USB_ENDPOINT_OUT+USB_TYPE_CLASS+USB_RECIP_INTERFACE;
    peticion.bRequest = DFU_DNLOAD;
    peticion.wValue = 0;
    peticion.wIndex = 0;
    peticion.wLength = 6;
    
    datos[0] = Id_display_data;
    datos[1] = 0x01;
    
    usbRequest (d, peticion, datos);
    /* Peticion blank check in flash */
    
    sendZLP(d);
    
    checkStatus (&estado, d);  
          
    sendZLP(d);
  
    if (estado.bStatus != OK) {
      return -2;
    } else {
      return 0;
    }
  }
}

void sendZLP (usb_dev_handle *d) {
  
  /*
    Envia una peticion ZLP
  */
  
  request peticion;
  unsigned char datos[2];
  
  peticion.bmRequestType = USB_ENDPOINT_OUT+USB_TYPE_CLASS+USB_RECIP_INTERFACE;
  peticion.bRequest = DFU_DNLOAD;
  peticion.wValue = 0;
  peticion.wIndex = 0;
  peticion.wLength = 0;
  
  usbRequest (d, peticion, datos);
  /* Envio de ZLP */
}
