//-----------------------------------------------------------------------------
//
//	This program is free software; you can redistribute it and/or
//	modify it under the terms of the GNU General Public License
//	as published by the Free Software Foundation; either version 2
//	of the License, or (at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//-----------------------------------------------------------------------------
// This file is "stolen" from the pk2 source code.
//
//#define USB_DEBUG
// Comment out the following line if you do not use usb hotplug
#define	USB_HOTPLUG

#include	<stdio.h>
#include <usb.h>			// libusb header
#include <unistd.h>		// for geteuid
#include	<ctype.h>
#include	<string.h>

#include	"bootUsb.h"

/* Stupidity:
   In an excellent example of sacrificing backward compatability
   for conformance to some proported "standard", the latest Linux 
   kernel USB drivers (uhci-alternate in 2.4.24+, and uhci in 2.6.x)
   no longer support low speed bulk mode transfers-- they give
   "invalid argument", errno=-22, on any attempt to do a low speed 
   bulk write.  Thus, we need interrupt mode transfers, which are 
   only available via the CVS version of libusb.
   
   (Thanks to Steven Michalske for diagnosing the true problem here.)
*/

#if HAVE_LIBUSB_INTERRUPT_MODE
// latest libusb: interrupt mode, works with all kernels
#  define DFU_USB(direction) usb_interrupt_##direction
#else 
// older libusb: bulk mode, will only work with older kernels
#  define DFU_USB(direction) usb_bulk_##direction
#endif

// Prototypes

// Data

usb_dev_handle	*deviceHandle = NULL;

int	dfu_interface = 0;
int	usbdebug = 0;

const static int endpoint_out = 1;		// endpoint 1 address for OUT
const static int endpoint_in = 0x81;	// endpoint 0x81 address for IN
const static int timeout = 15000;		// timeout in ms


int usbRequest (usb_dev_handle *d, request r, byte *data) {
  int n;
  n = usb_control_msg (d, r.bmRequestType, r.bRequest, r.wValue, r.wIndex, data, r.wLength, timeout);
  if (n < 0){
    perror ("Error setting USB request");
    return -1;
  } else {
    return n;
  }
}

void sendUSB( usb_dev_handle *d, const char *src, int len) {
  int	r, i;
  
  
  r = DFU_USB(write)(d, endpoint_out, (char *) src, reqLen, timeout);
  
}

int readBlock( usb_dev_handle *d, int len, byte *dest) {
  int	i, j, r;
  byte	bfr[reqLen + 1];
  
  r = DFU_USB(read)(d, endpoint_in, (char *) bfr, reqLen, timeout);
  
  /*   if (r != reqLen) */
  /*     printf("USB read did not return 64 bytes"); */
  
  if (usbdebug == 2)
    printf("USB<");
  
  for (i=0, j=0; i<len; i++){
    if (usbdebug == 2){
      printf(" %02x", bfr[i] & 0xff);
      j++;
      
      if ((j > 15) && (i < (len - 1))){
	j = 0;
	printf("\n    ");
      }
    }
    
    dest[i] = bfr[i];
  }
  
  if (usbdebug == 2)
    printf("\n");
  
  return len;
}

// Read this many bytes from this device

void recvUSB(usb_dev_handle *d, int len, byte *dest) {
  int r = readBlock(d, len, dest);
  
  if (r != len){
    
    perror("recvUSB() read error");
  }
}

// debugging: enable debugging error messages in libusb
#ifndef WIN32
extern int usb_debug;
#endif

// Find the first USB device with this vendor and product.
// Exits on errors, like if the device couldn't be found.
 
usb_dev_handle *deviceOpen(void) {
  
  int	result;
  struct usb_device	*device;
  struct usb_bus		*bus;
  
  usb_dev_handle *d = NULL;
  
  printf("\nSearching USB device (vendor 0x%04x/product 0x%04x)\n",
	 VENDOR_ID, PRODUCT_ID);
  // (libusb setup code stolen from John Fremlin's cool "usb-robot")
  
  usb_init();
  result = usb_find_busses();
  if (result < 0)
    printf("Error finding USB busses: %d\n", result);
  
  result = usb_find_devices();
  
  if (result < 0) 
    printf("Error finding USB devices: %d\n", result);
  
  
  
  for (bus = usb_get_busses(); bus != NULL; bus = bus->next) {
    struct usb_device* usb_devices = bus->devices;
    
    for (device = usb_devices; device != NULL; device = device->next) {
      if (device->descriptor.idVendor == VENDOR_ID
	  &&device->descriptor.idProduct == PRODUCT_ID){
	//	usb_dev_handle *d;
	
	printf( "Found AT90USB128x/AT90USB64x AVR USB Bootloader (DFU) as device '%s' on USB bus %s\n",
		device->filename,
		device->bus->dirname);
	
	d = usb_open(device);
	deviceHandle = d;
	
	if (d)	{			// This is our device
	  byte retData[reqLen + 1];
	  {
	    int retval;
	    char dname[32] = {0};
	    
	    retval = usb_get_driver_np(d, 0, dname, 31);
	    if (!retval)
	      usb_detach_kernel_driver_np(d, 0);
	  }
		  
	  //#ifdef CLAIM_USB
	  
	  if (usb_set_configuration(d, CONFIG_VENDOR)) {	// if config fails with CONFIG_VENDOR,
	    
	    if (usb_set_configuration(d, CONFIG_HID))	// it may be in bootloader, try CONFIG_HID
	      printf("Error setting USB configuration.\n");
	  }
	  
	  if (usb_claim_interface(d, dfu_interface)){
	    printf("Claim failed-- the USB AVR DFU is in use by another driver.\n"
		       "Do a `dmesg` to see which kernel driver has claimed it--\n"
		       "You may need to `rmmod hid` or patch your kernel's hid driver.\n");
	  }
	}
      }
    }
  }
  return d;
}

