/**
Intel MDS .hex file read and writer.

Orion Sky Lawlor, olawlor@acm.org, 2003/8/3

Modified by:
Mark Rages, markrages@gmail.com, 2005/04/01
Chen Xiao Fan, xiaofan@sg.pepperl-fuchs.com, 2005/08/12
Jeff Post, j_post@pacbell.net, 2005/10/16
Javier Almansa, javieralso@gmail.com, 2006/08/15
*/

#include <ctype.h>
#include <stdlib.h>
#include "hex.h"

/** Begin writing a .hex file here */
void hex_write_begin(FILE *f) {
  /* nothing to do, although you might set a type 02 or
     type 04 address marker here... */
}

typedef unsigned char byte;

/** Write this byte, updating checksum */
void hex_write_byte(FILE *f, unsigned int bi, byte *checksum) {
  byte b = (byte)(0xffu & bi);
  
  fprintf(f, "%02X", (int) b); 
  *checksum -= b;
}

/** Write this data to the .hex file at this address.
    Can write any number of bytes of data-- splits lines internally.
*/
void hex_write(FILE *f, unsigned int addr, unsigned int len, byte *data) {
#define hex_bytes_max 16
  if (len <= hex_bytes_max) { 
    /* Write out one line: */
    unsigned int i;
    
    byte checksum = (byte) 0;
    fprintf(f, ":");
    hex_write_byte(f, len, &checksum);
    hex_write_byte(f, addr >> 8, &checksum);
    hex_write_byte(f, addr, &checksum);
    hex_write_byte(f, 0x00u, &checksum);
    
    for (i=0; i<len; i++)
      hex_write_byte(f, data[i], &checksum);
    
    hex_write_byte(f, checksum, &checksum);
    fprintf(f, "\n"); 
  } else {
    /* len is too long-- split into smaller lines */
    
    while (len > 0) {
      int n = len;
      
      if (n > hex_bytes_max)
	n = hex_bytes_max;
      
      hex_write(f, addr, n, data);
      addr += n;
      len -= n;
      data += n;
    }
  }
}

/** Finish writing a .hex file here */
void hex_write_end(FILE *f) {
  fprintf(f, ":00000001FF\n");	/* special end marker */
}

/** Read a .hex file from here, sending the resulting address
    spans to this function. */
int hex_read(FILE *f, unsigned char *datos, unsigned short *addr, unsigned char *chk, unsigned short *len) {

  unsigned int i, type;
  byte checksum = 0u;

#define data_line_max 64u
      
  /* Seek to start of next line */
  int ic;
  
  while (EOF != (ic = fgetc(f))) {
    char c = (char) ic;
    
    if (c == ':')
      break; /* Start of next line */
/*     else if (!isspace(c)) */
/*       return -2; */
  }
      
  if (ic == EOF)
    return -3; /* Hit EOF */
      
  /* Read address and length of line */
  if (3 != fscanf(f, "%02x%04x%02x", len, addr, &type))
    return -4;
      
  checksum += *len;
  checksum += (*addr + (*addr >> 8));
  checksum += type;
      
  /* Read data and checksum for line: */
  if (*len > data_line_max)
    return -5;
  int c;
  c = *len;
  for (i=0; i<*len+1; i++) {
    if (1 != fscanf(f, "%02x", &datos[i]))
      return -6;
    checksum += datos[i];
  }
      
  if (0 != (checksum & 0xffu))
    return -7;
      
  /* Handle line */

  if (type == 0x00u || type == 0x01u || type == 0x02u || type == 0x04u || type == 0x03u){ 
    /* Regular data line-- pass data to user */
    *chk = checksum;

    return type;
  } else { 
    return -8; 
  }
}
